#!/usr/bin/env python
#-*- coding: utf-8 -*-
""" This daemon downloads and schedules the Series that dosage is tracking
because Bandwith is allways scarce there are some policies implemented in
order to improve the performance and make the dosage daemon the less invasive
posible.

The policies for downloading Tv Series are:

1) Never download More than one chapter per TV Series.

2) Allways download chapters with the max number of seeds.

3) Never download chapters with less than 5 seeds.

3) Only one Tv Series Can be at Junky mode at the time.

4) If 1 Tv Series is in Junky mode, don't download new chapters of tracking
   series

5) When you download the last chapter of a Tv Series on Junky mode, the junky
mode stops.

"""

import transmissionrpc
import mattdaemon
import sys
import logging
import sqlite3
import timeout_decorator

from urllib2 import URLError
from subprocess import Popen
from time import sleep
from dosagedb.models import *
from tpb import TPB
from tpb import ORDERS


logging.basicConfig()


class TorrentClient(object):

    def __init__(self):
        Popen(["transmission-gtk"])
        sleep(10)
        self.client = transmissionrpc.Client()

    def already_downloading(self, name):
        """Checks if at least one chapter is Downloading"""
        #Common torrent nomenclature replaces the space with a dot.
        name = name.replace(" ", ".")
        altname = name.replace(".", "+")
        for torrent in self.client.get_torrents():
            if name in torrent.name.lower() or altname in torrent.name.lower():
                if torrent.progress != 100:
                    return True
        return False

    def download(self, magnet):
        try:
            self.client.add_torrent(magnet)
        except transmissionrpc.error.TransmissionError:
            print("Already downloaded that torrent")


class TorrentProvider(object):

    def __init__(self):
        self.provider = TPB('https://thepiratebay.sx')
        self.MINIMUM_SEEDS = 5

    def find(self, seriename):
        return self._find(seriename)

    @timeout_decorator.timeout(10)
    def _find(self, seriename):
        print("Searching for %s" % seriename)
        search = self.provider.search(seriename)
        search.order(ORDERS.SEEDERS.ASC).multipage()
        try:
            torrent = next(search.items())
        except StopIteration:
            return None
        except URLError:
            print("Can't connect to the torrent provider")
            return None
        else:
            return torrent.magnet_link

class DosageDaemon(object):

    def __init__(self, client=TorrentClient, provider=TorrentProvider):
        self.client = client()
        self.provider = provider()

    def download(self, serie):
        #TODO: Crapy logic, make it better
        seriename, chapter, season = self.stringmaker(serie)

        try:
            torrent = self.provider.find(seriename)
        except timeout_decorator.TimeoutError:
            print("Timeout in the query to the provider")
            return

        #Lookup if a new season is available
        if not torrent:
            seriename, chapter, season = self.stringmaker(serie, newseason=1)
            try:
                torrent = self.provider.find(seriename)
            except timeout_decorator.TimeoutError:
                print("Timeout in the query to the provider")
                return

        if torrent:
            self.client.download(torrent)
            serie.last_chapter = chapter
            serie.last_season = season
            serie.save()

    def stringmaker(self, serie, newseason=0):
        if newseason == 1:
            chapter = str(1).zfill(2)
        else:
            chapter = str(serie.last_chapter + 1).zfill(2)
        season = str(serie.last_season + newseason).zfill(2)
        seriestring = serie.name + ' S' + season + 'E' + chapter
        return seriestring, chapter, season

    def run(self):
        serie = self.get_junky()
        if serie:
            serie = serie[0]  # Just One Junkie at the time Remember!
            if not self.client.already_downloading(serie.name):
                self.download(serie)
        else:
            series = self.get_tracking()
            for serie in series:
                if not self.client.already_downloading(serie.name):
                    self.download(serie)

    def get_junky(self):
        series = Series.select().where(Series.junkie == True)
        try:
            series[0]
        except IndexError:
            series = []
        return series

    def get_tracking(self):
        series = Series.select().where(Series.tracking == True)
        return series

    def already_downloading(self):
        pass


class MyDaemon(mattdaemon.daemon):

    def run(self, *args, **kwargs):
        dosage = DosageDaemon()
        while True:
            startdb()
            try:
                dosage.run()
            except sqlite3.InterfaceError:
                print("Sqlite Interface Error")
            closedb()
            sleep(60)

def main():
    args = {"pidfile": "/tmp/dosage-daemon.pid",
            "stdout": "/tmp/dosage-daemon.log",
            "stderr": "/tmp/dosage-daemon.log",
            "daemonize": True
            }
    daem = MyDaemon(**args)

    for arg in sys.argv[1:]:
        arg = arg.lower()
        if arg in ("-h", "--help"):
            print("python " + sys.argv[0] + " start|stop|restart|status|logs")
        elif arg in ("start", "start-no-daemon"):
            daem.start()
        elif arg in ("stop"):
            daem.stop()
        elif arg in ("restart"):
            daem.restart()
        elif arg in ("status"):
            if daem.status():
                print("dosage daemon currently running! :)")
            else:
                print("dosage daemon not running! :(")
                print("Check out the log in %s" % args["stderr"])
        elif arg in ("logs"):
            f = open(args['stderr'])
            log = f.read()
            print(log)
        elif arg in ("debug"):
            while True:
                daem.run()
        else:
            print("Unknown arg: %s" % arg)


if __name__ == "__main__":
    main()




#!/usr/bin/env python
#-*- coding: utf-8 -*-
"""This Module works as a Dosage Runner"""

import argparse
import logging

from dosagedb.models import *


logging.basicConfig()

parser = argparse.ArgumentParser()
parser.add_argument("-t", "--track", help="Start tracking the given tv series")
parser.add_argument("-u", "--untrack",
                    help="Stop tracking the given tv series")
parser.add_argument("-j", "--junky",
                    help="Put the given tv series on Junky mode")
parser.add_argument("-d", "--delete", help="Deletes a TV Series from the DB")
parser.add_argument("-s,", "--season", help="Declares a season Offset",
                    type=int, default=1)
parser.add_argument("-c,", "--chapter", help="Declares a Chapter Offset",
                    type=int, default=1)
parser.add_argument("-l,", "--list", help="list all the seriesi in the DB",
                    action="store_true")

#name = CharField(unique = True)
#last_season = IntegerField(default = 1, null = True)
#last_chapter = IntegerField(defautl = 1, null = True)
#tracking = BooleanField(default = False)
#junkie = BooleanField(default = False)
#quality =  CharField(default = "HDTV")


def track(name, season, chapter):
    name = name.lower()
    try:
        serie = Series.get(Series.name == name)
    except Series.DoesNotExist:
        serie = Series.create(name=name, last_season=season,
                              last_chapter=chapter, tracking=True)
    else:
        serie.tracking = True
        serie.save()
    print "Tracking %s" % name


def untrack(name):
    name = name.lower()
    try:
        serie = Series.get(Series.name == name)
    except Series.DoesNotExist:
        print "You are not Tracking that Tv Series"
    else:
        serie.tracking = False
        serie.junkie = False
        serie.save()
        print "Untracking %s" % name


def junky(name, season, chapter):
    name = name.lower()
    try:
        serie = Series.get(Series.name == name)
    except Series.DoesNotExist:
        serie = Series.create(name=name, last_season=season,
                              last_chapter=chapter, tracking=True, junkie=True)
    else:
        serie.tracking = True
        serie.junkie = True
        if serie.last_season <= season:
            serie.last_season = season
            if serie.last_chapter < chapter:
                serie.last_chapter = chapter
        serie.save()
    finally:
        #No more than 1 tv series in junky mode at the time
        series = Series.select().where(Series.name != name,
                                       Series.junkie == True)
        for serie in series:
            serie.junkie = False
            serie.save()
            print "%s is not more in Junky Mode" % serie.name
    print "%s is on Junky Mode" % name


def delete(name):
    name = name.lower()
    try:
        serie = Series.get(Series.name == name)
    except Series.DoesNotExist:
        print "There is not serie with that name on the DB"
    else:
        serie.delete_instance()
        print "%s has been removed from the DB" % name


def lister():
    series = Series.select()
    for serie in series:
        if serie.tracking:
            print "Tracking %s in Chapter %s and season %s" % \
                  (serie.name, str(serie.last_chapter),
                   str(serie.last_season))
        else:
            print "Not Tracking %s since Chapter %s and season %s" % \
                  (serie.name, str(serie.last_chapter),
                   str(serie.last_season))

def main():
    args = parser.parse_args()
    startdb()
    if args.track:
        track(name=args.track, chapter=args.chapter - 1,
              season=args.season)
    elif args.untrack:
        untrack(name=args.untrack)
    elif args.junky:
        junky(name=args.junky, chapter=args.chapter - 1,
              season=args.season)
    elif args.list:
        lister()
    elif args.delete:
        delete(name=args.delete)

if __name__ == "__main__":
    main()



from setuptools import setup, find_packages

setup(
        name='tvdosage',
        version='0.1.2',
        description='Keep your Tv Series dose going',
        long_description=(open('README.md').read()),
        url='http://github.com/jairot/dosage/',
        license='MIT',
        author='Jairo Trad',
        author_email='jtrad@insus.com.ar',
        packages = find_packages(exclude=['*_test.py']),
        py_modules = ['tvdosage'],
          entry_points={
                    'console_scripts':
                        ['dosagedaemon = scripts.dosagedaemon:main',
                         'tvdosage = scripts.tvdosage:main'
                         ]},
        install_requires=['ThePirateBay', 'argparse', 'beautifulsoup4', 'dateutils', 'lxml',
                          'mattdaemon','mccabe', 'peewee', 'purl', 'python-dateutil',
                          'pytz', 'six', 'timeout-decorator','transmissionrpc','wsgiref'],
        include_package_data=True,
        classifiers=[
                'Development Status :: 2 - Pre-Alpha',
                'Intended Audience :: End Users/Desktop',
                'Natural Language :: English',
                'License :: OSI Approved :: MIT License',
                'Operating System :: OS Independent',
                'Programming Language :: Python',
                'Programming Language :: Python :: 2.7',
            ],
    )




#!/urs/bin/env python
#-*- coding: utf-8 -*-

import os

from peewee import Model, SqliteDatabase
from peewee import CharField, IntegerField, BooleanField
from playhouse.proxy import Proxy
from sqlite3 import OperationalError

database_proxy = Proxy()


class CustomModel(Model):
    class Meta:
        database = database_proxy


class Series(CustomModel):
    name = CharField(unique=True)
    last_season = IntegerField(default=1, null=True)
    last_chapter = IntegerField(defautl=1, null=True)
    tracking = BooleanField(default=False)
    junkie = BooleanField(default=False)
    quality = CharField(default="HDTV")


def startdb(env='production'):
    if env == 'production':
        home = os.path.expanduser("~")
        dosagepath = os.path.join(home, ".tvdosage/")
        if not os.path.exists(dosagepath):
            os.makedirs(dosagepath)
        path = os.path.join(dosagepath,
                            'database.db')
        database = SqliteDatabase(path)
    elif env == 'testing':
        database = SqliteDatabase(':memory:')
    database_proxy.initialize(database)
    #Try to create a Table but if it exist fail silenty
    Series.create_table(fail_silently=True)

def closedb():
	database_proxy.close()

